(this.webpackJsonpstreamlit_component_template=this.webpackJsonpstreamlit_component_template||[]).push([[0],{5:function(e,t,n){"use strict";n.r(t);var o,a=n(1),s=n(4),i=n.n(s);n(12);a.a.events.addEventListener(a.a.RENDER_EVENT,(function(e){if(o)console.debug("skipped");else{var t=e.detail.args,n=t.config,s=t.interfaces,c=t.task;o=new i.a("root",{config:n,interfaces:s,task:c,onLabelStudioLoad:function(e){console.debug("loaded")},onSubmitAnnotation:function(e,t){console.debug("submitted"),t=JSON.parse(JSON.stringify(t)),a.a.setComponentValue(t)},onUpdateAnnotation:function(e,t){console.debug("updated"),t=JSON.parse(JSON.stringify(t)),a.a.setComponentValue(t),a.a.setFrameHeight()},onDeleteAnnotation:function(e,t){console.debug("deleted"),t=JSON.parse(JSON.stringify(t)),a.a.setComponentValue(t),a.a.setFrameHeight()}}),a.a.setFrameHeight()}})),a.a.setComponentReady()}},[[5,1,2]]]);
//# sourceMappingURL=main.7a994a46.chunk.js.map